import * as React from 'react';
import { TextControl } from '@tesseract-design/web-base';
import { delegateTriggerEvent, useClientSide } from '@modal-sh/react-utils';
import PhoneInput, { Country, Value } from 'react-phone-number-input/input';
import clsx from 'clsx';

export type PhoneNumberInputDerivedElement = HTMLInputElement;

export interface PhoneNumberInputProps extends Omit<React.HTMLProps<PhoneNumberInputDerivedElement>, 'size' | 'type' | 'label' | 'inputMode'> {
  /**
   * Short textual description indicating the nature of the component's value.
   */
  label?: React.ReactNode,
  /**
   * Short textual description as guidelines for valid input values.
   */
  hint?: React.ReactNode,
  /**
   * Size of the component.
   */
  size?: TextControl.Size,
  /**
   * Additional description, usually graphical, indicating the nature of the component's value.
   */
  indicator?: React.ReactNode,
  /**
   * Should the component display a border?
   */
  border?: boolean,
  /**
   * Should the component occupy the whole width of its parent?
   */
  block?: boolean,
  /**
   * Style of the component.
   */
  variant?: TextControl.Variant,
  /**
   * Is the label hidden?
   */
  hiddenLabel?: boolean,
  /**
   * Should the component be enhanced?
   */
  enhanced?: boolean,
  /**
   * Country where the phone number should be formatted for.
   */
  country?: Country,
}

/**
 * Component for inputting national and international phone numbers.
 */
export const PhoneNumberInput = React.forwardRef<
  PhoneNumberInputDerivedElement,
  PhoneNumberInputProps
>((
  {
    label,
    hint,
    indicator,
    size = 'medium' as const,
    border = false,
    block = false,
    variant = 'default' as const,
    hiddenLabel = false,
    className,
    id: idProp,
    style,
    enhanced = true,
    country = 'PH' as const,
    value,
    onChange,
    name,
    ...etcProps
  }: PhoneNumberInputProps,
  forwardedRef,
) => {
  const { clientSide } = useClientSide({ clientSide: enhanced });
  const [phoneNumber, setPhoneNumber] = React.useState<Value>(value?.toString() ?? '');
  const labelId = React.useId();
  const defaultId = React.useId();
  const id = idProp ?? defaultId;
  const defaultRef = React.useRef<PhoneNumberInputDerivedElement>(null);
  const ref = forwardedRef ?? defaultRef;

  const handlePhoneInputChange = (phoneNumberValue: Value) => {
    if (!(typeof ref === 'object' && ref)) {
      return;
    }
    const { current: input } = ref;
    if (!input) {
      return;
    }
    setPhoneNumber(phoneNumberValue);
    delegateTriggerEvent('change', input, phoneNumberValue ?? '');
  };

  const commonInputStyles = clsx(
    'bg-negative rounded-inherit w-full peer block font-inherit',
    'focus:outline-0',
    'disabled:opacity-50 disabled:cursor-not-allowed',
    {
      'text-xxs': size === 'small',
      'text-xs': size === 'medium',
    },
    {
      'pl-4': variant === 'default',
      'pl-1.5': variant === 'alternate',
    },
    {
      'pt-4': variant === 'alternate',
    },
    {
      'pr-4': variant === 'default' && !indicator,
      'pr-1.5': variant === 'alternate' && !indicator,
    },
    {
      'pr-10': indicator && size === 'small',
      'pr-12': indicator && size === 'medium',
      'pr-16': indicator && size === 'large',
    },
    {
      'h-10': size === 'small',
      'h-12': size === 'medium',
      'h-16': size === 'large',
    },
  );

  return (
    <div
      className={clsx(
        'relative rounded ring-secondary/50',
        'focus-within:ring-4',
        {
          'block': block,
          'inline-block align-middle': !block,
        },
        className,
      )}
      style={style}
    >
      <input
        {...etcProps}
        value={value}
        onChange={onChange}
        ref={ref}
        aria-labelledby={labelId}
        type="tel"
        id={id}
        name={name}
        data-testid="input"
        tabIndex={clientSide ? -1 : undefined}
        className={clsx(commonInputStyles, clientSide && 'sr-only')}
      />
      {clientSide && (
        <PhoneInput
          {...etcProps}
          ref={undefined}
          value={phoneNumber}
          onChange={handlePhoneInputChange}
          defaultCountry={country}
          className={commonInputStyles}
        />
      )}
      {label && (
        <label
          data-testid="label"
          id={labelId}
          htmlFor={id}
          className={clsx(
            'absolute z-[1] w-full top-0.5 left-0 pointer-events-none pl-1 text-xxs font-bold peer-disabled:opacity-50 peer-focus:text-secondary text-primary leading-none bg-negative select-none',
            {
              'sr-only': hiddenLabel,
            },
            {
              'pr-1': !indicator,
            },
            {
              'pr-10': indicator && size === 'small',
              'pr-12': indicator && size === 'medium',
              'pr-16': indicator && size === 'large',
            },
          )}
        >
          <span className="block w-full whitespace-nowrap h-[1.1em] overflow-hidden text-ellipsis">
            {label}
          </span>
        </label>
      )}
      {hint && (
        <div
          data-testid="hint"
          className={clsx(
            'absolute left-0 px-1 pointer-events-none text-xxs peer-disabled:opacity-50 leading-none w-full bg-negative select-none',
            {
              'bottom-0 pl-4 pb-1': variant === 'default',
              'top-0.5': variant === 'alternate',
            },
            {
              'pt-2': variant === 'alternate' && size === 'small',
              'pt-3': variant === 'alternate' && size !== 'small',
            },
            {
              'pr-4': !indicator && variant === 'default',
              'pr-1': !indicator && variant === 'alternate',
            },
            {
              'pr-10': indicator && size === 'small',
              'pr-12': indicator && size === 'medium',
              'pr-16': indicator && size === 'large',
            },
          )}
        >
          <div
            className="opacity-50 whitespace-nowrap w-full h-[1.1em] overflow-hidden text-ellipsis"
          >
            {hint}
          </div>
        </div>
      )}
      {indicator && (
        <div
          className={clsx(
            'text-center flex items-center justify-center peer-disabled:opacity-50 aspect-square absolute bottom-0 right-0 pointer-events-none select-none',
            {
              'w-10': size === 'small',
              'w-12': size === 'medium',
              'w-16': size === 'large',
            },
          )}
        >
          {indicator}
        </div>
      )}
      {border && (
        <span
          data-testid="border"
          className="absolute z-[1] peer-disabled:opacity-50 inset-0 rounded-inherit border-2 border-primary pointer-events-none peer-focus:border-secondary"
        />
      )}
    </div>
  );
});

PhoneNumberInput.displayName = 'PhoneNumberInput';

PhoneNumberInput.defaultProps = {
  label: undefined,
  hint: undefined,
  indicator: undefined,
  size: 'medium' as const,
  border: false as const,
  block: false as const,
  variant: 'default' as const,
  hiddenLabel: false as const,
  enhanced: false as const,
  country: 'PH' as const,
};
