import * as React from 'react';
import { TextControl } from '@tesseract-design/web-base';
import clsx from 'clsx';

export type TextInputDerivedElement = HTMLInputElement;

export interface TextInputProps extends Omit<React.HTMLProps<TextInputDerivedElement>, 'size' | 'type' | 'label' | 'list' | 'inputMode' | 'pattern'> {
  /**
   * Short textual description indicating the nature of the component's value.
   */
  label?: React.ReactNode,
  /**
   * Short textual description as guidelines for valid input values.
   */
  hint?: React.ReactNode,
  /**
   * Size of the component.
   */
  size?: TextControl.Size,
  /**
   * Additional description, usually graphical, indicating the nature of the component's value.
   */
  indicator?: React.ReactNode,
  /**
   * Should the component display a border?
   */
  border?: boolean,
  /**
   * Should the component occupy the whole width of its parent?
   */
  block?: boolean,
  /**
   * Type of the component value.
   */
  type?: TextControl.InputType,
  /**
   * Style of the component.
   */
  variant?: TextControl.Variant,
  /**
   * Is the label hidden?
   */
  hiddenLabel?: boolean,
  /**
   * Input mode of the component.
   */
  inputMode?: TextControl.InputMode,
}

/**
 * Component for inputting textual values.
 *
 * This component supports multiline input and adjusts its layout accordingly.
 */
export const TextInput = React.forwardRef<TextInputDerivedElement, TextInputProps>((
  {
    label,
    hint,
    indicator,
    size = 'medium' as const,
    border = false,
    block = false,
    type = 'text' as const,
    variant = 'default' as const,
    hiddenLabel = false,
    className,
    id: idProp,
    style,
    inputMode = type,
    ...etcProps
  },
  forwardedRef,
) => {
  const labelId = React.useId();
  const defaultId = React.useId();
  const id = idProp ?? defaultId;

  let resultInputMode = inputMode as React.HTMLProps<TextInputDerivedElement>['inputMode'];
  if (type === 'text' && resultInputMode === 'search') {
    resultInputMode = 'text';
  } else if (type === 'search' && resultInputMode === 'text') {
    resultInputMode = 'search';
  }

  return (
    <div
      className={clsx(
        'relative rounded ring-secondary/50 overflow-hidden',
        'focus-within:ring-4',
        {
          'block': block,
          'inline-block align-middle': !block,
        },
        className,
      )}
      style={style}
      data-testid="base"
    >
      <input
        {...etcProps}
        ref={forwardedRef}
        aria-labelledby={labelId}
        id={id}
        type={type}
        inputMode={resultInputMode}
        data-testid="input"
        className={clsx(
          'bg-negative rounded-inherit w-full peer block font-inherit',
          'focus:outline-0',
          'disabled:opacity-50 disabled:cursor-not-allowed',
          {
            'text-xxs': size === 'small',
            'text-xs': size === 'medium',
          },
          {
            'pl-4': variant === 'default',
            'pl-1.5': variant === 'alternate',
          },
          {
            'pt-4': variant === 'alternate',
          },
          {
            'pr-4': variant === 'default' && !indicator,
            'pr-1.5': variant === 'alternate' && !indicator,
          },
          {
            'pr-10': indicator && size === 'small',
            'pr-12': indicator && size === 'medium',
            'pr-16': indicator && size === 'large',
          },
          {
            'h-10': size === 'small',
            'h-12': size === 'medium',
            'h-16': size === 'large',
          },
        )}
      />
      {label && (
        <label
          data-testid="label"
          htmlFor={id}
          id={labelId}
          className={clsx(
            'absolute z-[1] w-full top-0.5 left-0 pointer-events-none pl-1 text-xxs font-bold peer-disabled:opacity-50 peer-focus:text-secondary text-primary leading-none bg-negative select-none',
            {
              'sr-only': hiddenLabel,
            },
            {
              'pr-1': !indicator,
            },
            {
              'pr-10': indicator && size === 'small',
              'pr-12': indicator && size === 'medium',
              'pr-16': indicator && size === 'large',
            },
          )}
        >
          <span className="block w-full whitespace-nowrap h-[1.1em] overflow-hidden text-ellipsis">
            {label}
          </span>
        </label>
      )}
      {hint && (
        <div
          data-testid="hint"
          className={clsx(
            'absolute left-0 px-1 pointer-events-none text-xxs peer-disabled:opacity-50 leading-none w-full bg-negative select-none',
            {
              'bottom-0 pl-4 pb-1': variant === 'default',
              'top-0.5': variant === 'alternate',
            },
            {
              'pt-2': variant === 'alternate' && size === 'small',
              'pt-3': variant === 'alternate' && size !== 'small',
            },
            {
              'pr-4': !indicator && variant === 'default',
              'pr-1': !indicator && variant === 'alternate',
            },
            {
              'pr-10': indicator && size === 'small',
              'pr-12': indicator && size === 'medium',
              'pr-16': indicator && size === 'large',
            },
          )}
        >
          <div
            className="opacity-50 whitespace-nowrap w-full h-[1.1em] overflow-hidden text-ellipsis"
          >
            {hint}
          </div>
        </div>
      )}
      {indicator && (
        <div
          data-testid="indicator"
          className={clsx(
            'text-center flex items-center justify-center peer-disabled:opacity-50 aspect-square absolute bottom-0 right-0 pointer-events-none select-none',
            {
              'w-10': size === 'small',
              'w-12': size === 'medium',
              'w-16': size === 'large',
            },
          )}
        >
          {indicator}
        </div>
      )}
      {border && (
        <span
          data-testid="border"
          className="absolute z-[1] peer-disabled:opacity-50 inset-0 rounded-inherit border-2 border-primary pointer-events-none peer-focus:border-secondary"
        />
      )}
    </div>
  );
});

TextInput.displayName = 'TextInput';

TextInput.defaultProps = {
  label: undefined,
  hint: undefined,
  size: 'medium',
  indicator: undefined,
  border: false,
  block: false,
  type: 'text',
  variant: 'default',
  hiddenLabel: false,
  inputMode: 'text',
};
